Index: modules/codec/ffmpeg/encoder.c
===================================================================
--- modules/codec/ffmpeg/encoder.c	(revision 20704)
+++ modules/codec/ffmpeg/encoder.c	(working copy)
@@ -835,6 +835,40 @@
     frame.pts /= p_enc->fmt_in.video.i_frame_rate;
     /* End work-around */
 
+    /* Dynamic aspect change check */
+    if( p_pict->format.i_aspect != p_enc->fmt_out.video.i_aspect )
+    {
+        int i_aspect_num, i_aspect_den;
+
+        msg_Dbg( p_enc, "EncodeVideo() p_pict: size %dx%d, aspect %d, sar %u:%u",
+                 p_pict->format.i_width, p_pict->format.i_height, p_pict->format.i_aspect,
+                 p_pict->format.i_sar_num, p_pict->format.i_sar_den );
+
+        p_enc->fmt_in.video.i_aspect =
+        p_enc->fmt_out.video.i_aspect =
+        p_pict->format.i_aspect;
+
+        p_enc->fmt_in.video.i_sar_num =
+        p_enc->fmt_out.video.i_sar_num =
+        p_pict->format.i_sar_num;
+
+        p_enc->fmt_in.video.i_sar_den =
+        p_enc->fmt_out.video.i_sar_den =
+        p_pict->format.i_sar_den;
+
+        av_reduce( &i_aspect_num, &i_aspect_den,
+                   p_pict->format.i_aspect,
+                   VOUT_ASPECT_FACTOR, 1 << 30 /* something big */ );
+
+        av_reduce( &p_sys->p_context->sample_aspect_ratio.num,
+                   &p_sys->p_context->sample_aspect_ratio.den,
+                   i_aspect_num * (int64_t)p_sys->p_context->height,
+                   i_aspect_den * (int64_t)p_sys->p_context->width, 1 << 30 );
+        msg_Dbg( p_enc, "AVCodecContext: %dx%d sar %d:%d",
+                 p_sys->p_context->width, p_sys->p_context->height,
+                 p_sys->p_context->sample_aspect_ratio.num, p_sys->p_context->sample_aspect_ratio.den );
+    }
+
     i_out = avcodec_encode_video( p_sys->p_context, (uint8_t*)p_sys->p_buffer_out,
                                   p_sys->p_context->height * p_sys->p_context->width * 3, &frame );
 
